	SUBROUTINE CALPAS(AGE,RELRISK,YEARS,LUNIT,FIXRSK,PREV)
	IMPLICIT NONE
C
C-Description-----------------------------------------------------------
C
C  Function:
C       Calculate values of PAS
C       PAS = p(a,s) in Dupont (1989) = the probability that the patient
C           develops the disease of interest by age A+S given that she
C           is alive and free of this disease at age A.  PAS is adjusted
C           for the possibility that the patient dies of other causes
C           before age A+S without ever developing the disease of interest.
C  Usage:
C       Absolute risk calculation program
C  Arguments:
C   i   AGE     (Real)          patient's age
C   i   RELRISK (Real)          patient's relative risk
C   i   YEARS   (Real)          years over which risk is to be estimated
C   i   LUNIT   (Integer)       Log file unit number
C   i   FIXRSK  (Char*1)
C   i   PREV    (Real)
C  Notes:
C   .   This routine was written by Dale Plummer
C
C-Declarations----------------------------------------------------------
C
	INCLUDE 'ABSRISK.INC'
C
C  Arguments
C
	INTEGER LUNIT
	REAL RELRISK,AGE,YEARS,PREV
	CHARACTER FIXRSK*1
C
C  Functions
C
	REAL INTEG,INTERP,RRISK
	DOUBLE PRECISION SIMINT
C
C  Locals
C
	EXTERNAL IGRAND
	INTEGER NN
	PARAMETER (NN=2)
	REAL ERROR
	PARAMETER (ERROR=.005)
	INTEGER S,I,T0,J,ENDAGE
	REAL L0(200),M0(200),PAS,RR,AA,PAS1,P
C
C
C-Code------------------------------------------------------------------
C
C    Copy argument to local storage.
C
	RHO=RELRISK
C
C    Adjust relative risk if called for.
C
	IF (FIXRSK.EQ.'Y') THEN
	    P=PREV/100.
	    RHO=RHO/(1.+P*(RHO-1.))
	END IF
C
C    Set entry age.
C
	A=AGE
C
C    Do the calculations with the constant hazard function
C
	WHICHF=1
C
C
	DO ENDAGE=A,105
	    L0(ENDAGE)=INTERP(ENDAGE,C,W,FAGE)
	    M0(ENDAGE)=INTERP(ENDAGE,DMINUS,W,FAGE)
	    RR=RRISK(RHO,REAL(ENDAGE),REAL(A),WHICHF)
	    TOTALH(ENDAGE)=(RR*L0(ENDAGE))+M0(ENDAGE)
	END DO
	CTOTLH(0)=0
	T0=A
	DO I=1,105-A
	    CTOTLH(I)=CTOTLH(I-1)+INTEG(TOTALH(T0),TOTALH(T0+1),1.)
	    T0=T0+1
	END DO
C
C    Initialize.
C
	PAS=0
	AA=A
C
C    Constant hazard.
C
	S=0
	DO J=1,YEARS
	    DA=AA
	    DAS=AA+1
	    PAS1=SIMINT(IGRAND,DA,DAS,ERROR,0.1E-15,
     .                  NN,KOUNT,IQUIT,HMAX,HMIN)
	    PAS=PAS+PAS1
	    S=S+(DAS-DA)
	    AA=DAS
	END DO
C
C    Write output.
C
	WRITE(UNIT=6,FMT='(/A,I3,A,F5.3)')
     .  ' Probability that patient will suffer outcome by age ',
     .  INT(AGE+YEARS),' = ',PAS
	WRITE(UNIT=LUNIT,FMT='(/A,I3,A,F5.3)')
     .  ' Probability that patient will suffer outcome by age ',
     .  INT(AGE+YEARS),' = ',PAS
C
C    Finished.
C
	RETURN
	END
