C****************************SIMINT*************************************!0129200
C                                                                       !0129300
C  THIS IS AN AUTOMATIC INTEGRATION PROGRAM USING                       !0129400
C  A CORRECTED SIMPSON'S RULE AND FOURTH DIVIDED DIFFERENCES            !0129500
C  TO ESTIMATE THE FOURTH DERIVATIVE AND HENCE THE                      !0129600
C  ERROR BOUND FOR EACH PAIR OF SIMPSON'S APPROXIMATIONS                !0129700
C                                                                       !0129800
C  USEAGE                                                               !0129900
C                                                                       !0130000
C     RESULT=SIMINT(FCN,A,B,ERROR,RNDER,N,KOUNT,IQUIT,HMAX,HMIN)        !0130100
C                                                                       !0130200
C  DESCRIPTION OF PARAMETERS                                            !0130300
C                                                                       !0130400
C     FCN IS THE FUNCTION TO BE INTEGRATED                              !0130500
C     A IS THE BEGINNING OF THE INTERVAL OF INTEGRATION                 !0130600
C     B IS THE END OF THE INTERVAL OF INTEGRATION                       !0130700
C     ERROR IS THE TOTAL ALLOWABLE ERROR                                !0130800
C     RNDER IS THE MAXIMUM ABSOLUTE                                     !0130900
C       ROUND OFF ERROR IN FCN(X) FOR X IN (A,B)                        !0131000
C     N IS AN INTEGER WHICH LIMITS                                      !0131100
C       THE MAXIMUM INTERVAL SIZE TO 2.**-N                             !0131200
C     KOUNT RETURNS THE NUMBER OF EVALUATIONS OF FCN                    !0131300
C     IQUIT RETURNS THE NUMBER OF INTERVALS ON WHICH                    !0131400
C      THE REQUIRED ERROR BOUND WAS NOT ATTAINED                        !0131500
C     HMAX  RETURNS THE MAXIMUM INTERVAL SIZE USED                      !0131600
C     HMIN  RETURNS THE MINIMUM INTERVAL SIZE USED                      !0131700
C                                                                       !0131800
C  REMARKS                                                              !0131900
C                                                                       !0132000
C     FCN MUST BE EXTERNALED IN THE CALLING PROGRAM                     !0132100
C     ERROR MUST BE GREATER THAN RNDER*(B-A)                            !0132200
C     A, B, AND FCN MUST BE DOUBLE PRECISION                            !0132300
C     ERROR AND RNDER MUST BE REAL                                      !0132400
C     RNDER MUST BE CHOSEN SO THAT ABS( RNDER/FCN(X) ) IS GREATER THAN  !0132500
C                                            .5*E-17 FOR ALL X          !0132600
C       FUNCTIONS REFERENCED:                                           !0132700
C                                                                       !0132800
C               FCN                                                     !0132900
C                                                                       !0133000
C                                                                       !0133100
C  PROGRAMED BY                                                         !0133200
C                                                                       !0133300
C                            W. D. DUPONT                               !0133400
C                            MCGILL COMPUTING CENTER                    !0133500
C                            SUMMER 1970                                !0133600
C                                                                       !0133700
C***********************************************************************!0133800
        DOUBLE PRECISION FUNCTION SIMINT                                !0133900
     +        (FCN,A,B,ERROR,RNDER,NN,KOUNT,IQUIT,HMAX,HMIN)            !0134000
CVAX      IMPLICIT NONE                                                 !0134100
        INTEGER         IQUIT,          KOUNT,          LOCAL           !0134200
     +,                 N,              NN                              !0134300
        REAL            BOUND,          ERROR,          HMAX            !0134400
     +,                 HMIN,           RNDER,          SUBER           !0134500
     +,                 SUM,            TRNKER                          !0134600
        DOUBLE PRECISION                A,              B               !0134700
     +,                 C,              DIF,            FCN             !0134800
     +,                 FUNC(121),      H,              SIGN            !0134900
     +,                 VALUE(121)                                      !0135000
        SIGN =0.1D1                                                     !0135100
        IF (B.GE.A)                                     GO TO 1         !0135200
        C = A                                                           !0135300
        A = B                                                           !0135400
        B = C                                                           !0135500
        SIGN=    -0.1D1                                                 !0135600
C                                                                       !0135700
C  TEST WHETHER ROUND OFF ERROR IS                                      !0135800
C       SUFFICIENTLY SMALL TO GIVE REQUIRED ACCURACY                    !0135900
C                                                                       !0136000
1       IF (ERROR.LE.RNDER*(B-A))                               GO TO 12!0136100
        SIMINT=0.0D1                                                    !0136200
        IF ( A.EQ.B)                                    RETURN          !0136300
        TRNKER=(ERROR  -RNDER*(B-A))/2.0                                !0136400
        N=NN                                                            !0136500
        VALUE(117)=A                                                    !0136600
        FUNC(117)=FCN(A)                                                !0136700
        KOUNT = 1                                                       !0136800
        I QUIT =0                                                       !0136900
        HMAX=  0.0D0                                                    !0137000
        HMIN = 0.2D1**(-N)                                              !0137100
C                                                                       !0137200
C  TEST WHETHER 2**(-N) IS TOO LARGE                                    !0137300
C                                                                       !0137400
25      IF (A+0.4D1*0.2D1**(-N) .LE.B        )          GO TO 21        !0137500
        N=N+1                                                           !0137600
                                                        GO TO 25        !0137700
C                                                                       !0137800
C  CHOOSE NEXT BLOCK OF SIZE 2**(-N)*4.                                 !0137900
C                                                                       !0138000
21      H=0.2D1**(-N)                                                   !0138100
        VALUE (121) = VALUE(117) +0.4D1*H                               !0138200
500     SUBER=TRNKER*(H*4.0 /(B-A))                                     !0138300
        VALUE(120) = VALUE(117) +0.3D1*H                                !0138400
        VALUE(119) = VALUE(117) +0.2D1*H                                !0138500
        VALUE(118) = VALUE(117) +0.1D1*H                                !0138600
        FUNC(121) = FCN(VALUE(121))                                     !0138700
        FUNC(120) = FCN(VALUE(120))                                     !0138800
        FUNC(119) = FCN(VALUE(119))                                     !0138900
        FUNC(118) = FCN(VALUE(118))                                     !0139000
        SUM = DABS(FUNC(117)) + DABS(FUNC(118)) + DABS(FUNC(119))       !0139100
     +        + DABS(FUNC(120)) + DABS(FUNC(121))                       !0139200
        BOUND= ERROR /(2.0*SUM*(B-A)+1.0)                               !0139300
        LOCAL=119                                                       !0139400
C                                                                       !0139500
C  CALCULATE ESTIMATE OF ERROR BOUND, BRANCH TO 11 IF                   !0139600
C  ESTIMATE IS SUFFICIENTLY SMALL                                       !0139700
C                                                                       !0139800
10      DIF=(H/0.45D2)*(FUNC(LOCAL-2)-0.4D1*FUNC(LOCAL-1)      +        !0139900
     +   0.6D1*FUNC(LOCAL)-0.4D1*FUNC(LOCAL+1)        +FUNC(LOCAL+2))   !0140000
        IF (  SUBER .GE. DABS(DIF))                     GO TO 11        !0140100
        IF (H.LE.BOUND)                                 GO TO 111       !0140200
        IF (LOCAL - 4.LE.0)                             GO TO 111       !0140300
C                                                                       !0140400
C  DIVIDE INTERVAL BY 2 AND CALCULATE NEW VALUES                        !0140500
C                                                                       !0140600
        H=H/0.2D1                                                       !0140700
        SUBER = SUBER/2.0                                               !0140800
        LOCAL=LOCAL-2                                                   !0140900
13      VALUE(LOCAL-2)=VALUE(LOCAL)                                     !0141000
        FUNC(LOCAL-2)=FUNC(LOCAL)                                       !0141100
        VALUE(LOCAL)=VALUE(LOCAL+1)                                     !0141200
        FUNC(LOCAL)=FUNC(LOCAL+1)                                       !0141300
        VALUE  (LOCAL-1)  =  VALUE(LOCAL)  -H                           !0141400
        VALUE  (LOCAL+1)  =  VALUE(LOCAL)  +H                           !0141500
        FUNC(LOCAL-1)  =  FCN (  VALUE(  LOCAL-1))                      !0141600
        FUNC(LOCAL+1)  =  FCN (  VALUE(  LOCAL+1))                      !0141700
                                                        GO TO 10        !0141800
C                                                                       !0141900
C  CALCULATE INTEGRAL ESTIMATE&ON INTERVAL                              !0142000
C                                                                       !0142100
111     SIMINT = SIMINT + H*(FUNC(LOCAL-2)/0.2D1+ FUNC(LOCAL-1)         !0142200
     +     + FUNC(LOCAL) + FUNC(LOCAL+1) + FUNC(LOCAL+2)/0.2D1)         !0142300
        I QUIT = I QUIT +1                                              !0142400
                                                        GO TO 112       !0142500
11      SIMINT=SIMINT + (H/0.3D1)*(FUNC(LOCAL-2)+0.4D1*FUNC(LOCAL-1) +  !0142600
     +   0.2D1*FUNC(LOCAL)  + 0.4D1*FUNC(LOCAL + 1) +                   !0142700
     +        FUNC(LOCAL +2))-DIF                                       !0142800
112     LOCAL=LOCAL + 2                                                 !0142900
        IF ( H.GT.HMAX) HMAX = H                                        !0143000
        IF (H .LT. HMIN ) HMIN = H                                      !0143100
        KOUNT=KOUNT+4                                                   !0143200
        IF  (  LOCAL  .GE.121 )                         GO TO 20        !0143300
C                                                                       !0143400
C  CHOOSE NEXT  INTERVAL                                                !0143500
C                                                                       !0143600
        H  =  (  VALUE(LOCAL+1)-VALUE(LOCAL  ))/0.2D1                   !0143700
        SUBER=(4.0*H*TRNKER)/(B-A)                                      !0143800
                                                        GO TO 13        !0143900
12      WRITE(9, 3) ERROR                                               !0144000
                                                    STOP                !0144100
20      IF (VALUE(121)   .EQ. B)                                GO TO 2 !0144200
        VALUE(117)= VALUE(121)                                          !0144300
        FUNC(117)=FUNC(121)                                             !0144400
C                                                                       !0144500
C  TEST WHETHER 2**(-N) IS TOO LARGE                                    !0144600
C                                                                       !0144700
        IF (VALUE(117) + 0.4D1   *0.2D1**(-N).LE.B)     GO TO 21        !0144800
        IF (SUBER.GE.4.0*DABS(DIF))                     GO TO 24        !0144900
26      N=N+1                                                           !0145000
        IF (VALUE(117) + 0.4D1 * 0.2D1**(-N)-B)         21,21,26        !0145100
24      H=(B-VALUE(117))/0.4D1                                          !0145200
        VALUE(121) = B                                                  !0145300
                                                        GO TO 500       !0145400
2       IF (SIGN.GE.0.0D0)                              RETURN          !0145500
        B=A                                                             !0145600
        A=C                                                             !0145700
        SIMINT=SIMINT*SIGN                                              !0145800
                                                        RETURN          !0145900
3       FORMAT(    40H0   THE ROUND OFF ERROR PER TERM IS TOO           !0146000
     + ,40HLARGE TO ALLOW AN INTEGRAL APPROXIMATION                     !0146100
     + ,28H WITH TOTAL ERROR LESS THAN  ,E12.4)                         !0146200
        END                                                             !0146300
